﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ClasterCollection: TemplatedList<ClasterCollection, Claster> {

		internal ClasterCollection() : base() { }

		public Claster this[ string iSymbol ] {
			get {
				Claster _Next = this.First;
				Claster _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Symbol == iSymbol )
						return _Item;
				}
				return null;
			}
		}
		
		public Claster this[ Context iContext ] {
			get {
				Claster _Next = this.First;
				Claster _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item == iContext.Claster )
						return _Item;
				}
				return null;
			}
		}

		public Claster Append( Context iContext, string iSymbol, int iDigits, int iSpread, double iPoint ) {
			if ( iContext == null )
				throw new ArgumentNullException( "iContext" );
			if ( iContext.Claster != null )
				throw new ArgumentException( "Claster is defined", "iContext" );
			if ( iSymbol == null )
				throw new ArgumentNullException( "iSymbol" );
			Claster _Item = this[ iSymbol ];
			if ( _Item == null ) {
				_Item = AddFinal( new Claster( iSymbol, iDigits, iSpread, iPoint ) );
			} else {
				_Item.Digits = iDigits;
				_Item.Spread = iSpread;
				_Item.Point = iPoint;
			}
			_Item.Session.Append( iContext );
			return _Item;
		}

		public Claster Remove( Context iContext ) {
			if ( iContext == null )
				throw new ArgumentNullException( "iContext" );
			if ( iContext.Claster == null )
				throw new ArgumentException( "Claster not defined", "iContext" );
			Claster _Item = this[ iContext ];
			_Item.Session.Remove( iContext );
			if ( _Item.Session.Count == 0 ) {
				Remove( _Item );
			}
			return _Item;
		}

		public Claster Remove( string iSymbol ) {
			if ( iSymbol == null )
				throw new ArgumentNullException( "iSymbol" );
			Claster _Item = this[ iSymbol ];
			if ( _Item != null ) {
				Remove( _Item );
			}
			return _Item;
		}
		
		public Claster[] Array {
			get {
				Claster[] _Array = new Claster[ Count ];
				int _Index = 0;
				Claster _Item = this.First;
				while ( _Item != null ) {
					_Array[ _Index ] = _Item;
					_Item = _Item.Next;
					_Index++;
				}
				return _Array;
			}
		}
		
	}

	public class Claster: TemplatedItem<ClasterCollection, Claster> {

		public readonly string Symbol;
		public readonly SessionCollection Session;
		public readonly TickCollection Ticks;
		
		public int Digits;
		public int Spread;
		public double Point;

		internal CompareMono Up;
		internal CompareMono Down;

		internal Claster( string iSymbol, int iDigits, int iSpread, double iPoint )
			: base() {
			Symbol = iSymbol;
			Session = new SessionCollection( this );
			Ticks = new TickCollection( this, -1 );
			Digits = iDigits;
			Spread = iSpread;
			Point = iPoint;
			Up = null;
			Down = null;
		}

		public override void OnAdded() {
			Global.Compare.AppendClaster( this );
		}
		

		public override void OnRemoved() {
			Global.Compare.RemoveClaster( this );
			Session.Clear();
		}

	}

}