﻿/******************************************************************************\
   Все права (c) XNSNET 2005 - 2007 & Автор Иванов M.A. [Shum.er] 2005 - 2007 
\******************************************************************************/
namespace XNSNET.Collections {

	#region :- Импорт -:

	using System;

	#endregion :- Импорт -:

	#region :комменты:
	/// <summary>
	/// Астрактный класс шаблоноого списка связанных элементов.
	/// </summary>
	/// <typeparam name="LIST"> Шаблонный тип списка связанных элементов. </typeparam>
	/// <typeparam name="ITEM"> Шаблонный тип элемента связанного списка. </typeparam>
	/// <example>
	/// </example>
	/// <remarks>
	/// Используется для элементарного связывания элементов списка, где все элементы
	/// включая сам список между собой взаимосвязаны, без дополнительного распределения
	/// новых блоков памяти для включения в список новых элементов или для удаления.
	/// </remarks>
	#endregion :комменты:
	public abstract class TemplatedList<LIST, ITEM>
		where LIST: TemplatedList<LIST, ITEM>
		where ITEM: TemplatedItem<LIST, ITEM> {
	
    private int mCount;
		private ITEM mFirst;
		private ITEM mFinal;

		protected TemplatedList() {
      Clearing();
    }

    private void Clearing() {
      mCount = 0;
      mFirst = null;
      mFinal = null;
    }

    public int Count {
      get {
        return mCount;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value < 1 || value == mCount )
          throw new ArgumentOutOfRangeException();
#endif
        #endregion #исключения#
        mCount = value;
      }
    }

		public ITEM First {
      get {
        return mFirst;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value == null || value == mFirst )
          throw new ArgumentNullException();
#endif
        #endregion #исключения#
        mFirst = value;
      }
    }

		public ITEM Final {
      get {
        return mFinal;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value == null || value == mFinal )
          throw new ArgumentNullException();
#endif
        #endregion #исключения#
        mFinal = value;
      }
    }

		public virtual void Clear() {
      if ( this.Count < 1 )
        return;
      lock ( this ) {
				ITEM _Next = this.First;
				ITEM _Item;
        while ( _Next != null ) {
          _Item = _Next;
          _Next = _Next.Next;
          _Item.Invalidate();
          _Item.OnRemoved();
        }
        this.Clearing();
      }
    }

		protected ITEM AddFinal( ITEM iItem ) {
      if ( iItem == null )
        throw new ArgumentNullException();
			if ( !iItem.IsInvalid() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = (LIST)this;
        if ( this.Count == 0 ) {
          this.First = iItem;
        } else {
          iItem.Prev = this.Final;
          this.Final.Next = iItem;
        }
        this.Final = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected ITEM AddFirst( ITEM iItem ) {
      #region #исключения#
#if DEBUG
      if ( iItem == null )
        throw new ArgumentNullException();
			if ( !iItem.IsInvalid() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (LIST)this;
        if ( this.Count == 0 ) {
          this.Final = iItem;
        } else {
          iItem.Next = this.First;
          this.First.Prev = iItem;
        }
        this.First = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected ITEM AddAfter( ITEM iTodo, ITEM iItem ) {
      #region #исключения#
#if DEBUG
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
			if ( iTodo.IsInvalid() || !iItem.IsInvalid() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (LIST)this;
        iItem.Prev = iTodo;
        if ( iTodo.Next == null ) {
          this.Final = iItem;
        } else {
          iItem.Next = iTodo.Next;
          iTodo.Next.Prev = iItem;
        }
        iTodo.Next = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected ITEM AddBefore( ITEM iTodo, ITEM iItem ) {
      #region #исключения#
#if DEBUG
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
			if ( iTodo.IsInvalid() || !iItem.IsInvalid() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (LIST)this;
        iItem.Next = iTodo;
        if ( iTodo.Prev == null ) {
          this.First = iItem;
        } else {
          iItem.Next = iTodo;
          iTodo.Prev.Next = iItem;
        }
        iTodo.Prev = iItem;
        this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		public virtual ITEM Remove( ITEM iItem ) {

      #region #исключения#
#if DEBUG
      if ( iItem == null )
        throw new ArgumentNullException();
#endif
      #endregion #исключения#

      if ( this.Count == 1 ) {
        this.Clearing();
      } else {
				lock ( this ) {
					if ( iItem.Prev != null && iItem.Next != null ) {
						iItem.Prev.Next = iItem.Next;
						iItem.Next.Prev = iItem.Prev;
					} else if ( iItem.Prev == null ) {
						this.First = iItem.Next;
						iItem.Next.Prev = null;
					} else if ( iItem.Next == null ) {
						this.Final = iItem.Prev;
						iItem.Prev.Next = null;
					}
					Count--;
        }
      }
			iItem.OnRemoved();
      iItem.Invalidate();
			return iItem;

    }

		//public ITEM this[ int iIndex ] {
		//  get {
		//    string place = null;
		//    lock ( this ) {
		//      ITEM _Next = this.First;
		//      ITEM _Item;
		//      int _Index = 0;
		//      while ( _Next != null ) {
		//        _Item = _Next;
		//        _Next = _Next.Next;
		//        if ( _Index == iIndex )
		//          return _Item;
		//        _Index++;
		//      }
		//    }
		//    return null;
		//  }
		//}
		
	}

}