﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class CompareMonoCollection: TemplatedList<CompareMonoCollection, CompareMono> {

		internal CompareMonoCollection() : base() {
			CompareMono _USD = Append( "USD" );
			CompareMono _EUR = Append( "EUR" );
			CompareMono _JPY = Append( "JPY" );
			CompareMono _CHF = Append( "CHF" );
			CompareMono _GBP = Append( "GBP" );
		}

		public CompareMono this[ string iName ] {
			get {
				CompareMono _Next = this.First;
				CompareMono _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Name == iName )
						return _Item;
				}
				return null;
			}
		}

		public CompareMono Append( string iName ) {
			if ( iName == null )
				throw new ArgumentNullException();
			CompareMono _Item = this[ iName ];
			if ( _Item != null ) return _Item;
			return AddFinal( new CompareMono( iName ) );
		}

		public CompareMono Remove( string iName ) {
			if ( iName == null )
				throw new ArgumentNullException();
			CompareMono _Item = this[ iName ];
			if ( _Item != null ) {
				Remove( _Item );
			}
			return _Item;
		}
		
		public void AppendClaster( Claster iClaster ) {
			if ( iClaster == null ) return;
			if ( iClaster.Up != null && iClaster.Down != null ) return;
			CompareMono _Next = this.First;
			CompareMono _Item;
			CompareMono _Up = null;
			CompareMono _Down = null;
			while ( _Next != null && ( _Up == null || _Down == null ) ) {
				_Item = _Next;
				_Next = _Next.Next;
				if ( iClaster.Symbol.StartsWith( _Item.Name ) ) {
					_Up = _Item;
				} else if ( iClaster.Symbol.EndsWith( _Item.Name ) ) {
					_Down = _Item;
				}
			}
			if ( _Up != null && _Down != null ) {
				_Down.Pairs.Append( _Up ).Claster = iClaster;
				_Up.Pairs.Append( _Down ).Claster = iClaster;
				iClaster.Up = _Up;
				iClaster.Down = _Down;
			}
		}

		public void RemoveClaster( Claster iClaster ) {
			if ( iClaster == null ) return;
			if ( iClaster.Up == null && iClaster.Down == null ) return;
			CompareMono _Up = iClaster.Up;
			CompareMono _Down = iClaster.Down;
			_Up.Pairs.Remove( _Down );
			_Down.Pairs.Remove( _Up );
			iClaster.Up = null;
			iClaster.Down = null;
		}

		public CompareMono[] Array {
			get {
				CompareMono[] _Array = new CompareMono[ Count ];
				int _Index = 0;
				CompareMono _Item = this.First;
				while ( _Item != null ) {
					_Array[ _Index ] = _Item;
					_Item = _Item.Next;
					_Index++;
				}
				return _Array;
			}
		}

	}

	public class CompareMono: TemplatedItem<CompareMonoCollection, CompareMono> {

		public readonly string Name;
		public ComparePairCollection Pairs;

		internal CompareMono( string iName )
			: base() {
			Name = iName;
			Pairs = new ComparePairCollection( this );
		}

		public override void OnRemoved() {
			Pairs.Clear();
			CompareMono _Next = Global.Compare.First;
			CompareMono _Item;
			while ( _Next != null ) {
				_Item = _Next;
				_Next = _Next.Next;
				_Item.Pairs.Remove( this );
			}
		}

		public override string ToString() {
			return Name;
		}

	}

}